using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using PDFDecNetPro;

namespace Sharp_PDF_EP
{
    public partial class Form1 : Form
    {
        private Bitmap Picture;
        private Image SourcePicture=null;
        private EncodingInfo[ ] Info;
        private int CodePage;
        private bool optionsChanged = true;
        private int symbolIndex = -1;
        
        private TPDF417_Decoder Decoder = null;
        
        private void InitializeCodePages ( )
        {
            Info = Encoding.GetEncodings( );

            for ( int i = 0; i < Info.Length; i++ )
            {
                String S;
                Encoding E = Console.Out.Encoding;
                int CP = Info[ i ].CodePage;
                S = CP.ToString( ) + " - " + Info[ i ].DisplayName;
                comboBox1.Items.Add( S );
                if ( E.CodePage == CP )
                {
                    comboBox1.SelectedIndex = i;
                    CodePage = CP;
                }
            }
        }
        public Form1 ()
        {
            InitializeComponent( );
            InitializeCodePages( );

            SpeedComboBox.SelectedIndex = 1;
            DirectComboBox.SelectedIndex = 0;
            QualityComboBox.SelectedIndex = 0;
            bwFilterComboBox.SelectedIndex = 1;
            QualityComboBox.SelectedIndex = 1;
        }
  
        private void DrawCorners (bool Ok, int[] RowCols)
        {
            if ( Picture == null ) return;
            float X1, X2, Y1, Y2;
            int I1, I2, I;
            Pen P;
            float wid = Picture.Width/600+3;
            if ( Ok )
                P = new Pen( Color.Lime,wid);
            else
                P = new Pen(Color.Red, wid);
            Graphics G = Graphics.FromImage(Picture);
            for ( I = 0; I < 4; I++ )
            {
                I1 = ( I * 2 + 0 );
                I2 = ( I * 2 + 2 );
                if ( I2 > 7 ) I2 = 0;
                X1 = (float)RowCols[ I1 + 1 ];
                Y1 = (float)RowCols[ I1 + 0 ];
                X2 = (float)RowCols[ I2 + 1 ];
                Y2 = (float)RowCols[ I2 + 0 ];
                G.DrawLine( P, X1, Y1, X2, Y2 );
            }

            Img.Image = Picture;
        }

        private void SetOptions()
        {
           if (!optionsChanged) return;
           Decoder.SetMaxSymbolCount((int)MaxPDFCountEdit.Value);
           
           Decoder.SetSpeedMode((TPDF417_SpeedMode)SpeedComboBox.SelectedIndex);
           
           Decoder.SetDirectMode((TPDF417_DirectMode)DirectComboBox.SelectedIndex);
           
           Decoder.SetScanStep((int)stepNumericUpDown1.Value);
           
           if (QualityComboBox.SelectedIndex == 1)
              Decoder.SetQualityOn(true);
           else
              Decoder.SetQualityOn(false);

           Decoder.SetBWFilter(Convert.ToInt32(bwFilterComboBox.Text));

           Decoder.SetTmeOut((int)TimeOutEdit.Value);
           optionsChanged = false;
        }


        private void button2_Click (object sender, EventArgs e)
        {
            ClearForm();
            
            Img.Image = SourcePicture;
            if (Picture != null)
            {
               Picture.Dispose();
               Picture = null;
            }
            Picture = new Bitmap(SourcePicture);

            symbolIndex = 0;
            DateTimeOffset dto;
            int ms = 0;
            int s = 0;
            int h = 0, m = 0;

            try
            {
               if (Decoder == null)
                  Decoder = new TPDF417_Decoder(4000, 4000);

               SetOptions();
               
               dto = DateTimeOffset.Now;
               ms = dto.Millisecond;
               h = dto.Hour;
               m = dto.Minute;
               s = dto.Second;

               Decoder.DecodeImage(Picture);
            }catch(Exception ex)
            {
               MessageBox.Show(ex.Message);
            }

            dto = DateTimeOffset.Now;
            h = dto.Hour - h;
            m = dto.Minute - m;
            s = dto.Second - s;
            ms = dto.Millisecond - ms;

            if (((h * 12 + m) * 60 + s) < 0)
                h = m = s = ms = 0;
            TimeEdit.Text = (((h*12+m)*60+s)*1000+ms).ToString();


            if (Decoder.RejectionReasons() == TPDF_Rejection_Reasons.PDF417_OK)
                rrEdit.Text = "OK";
            else if (Decoder.RejectionReasons() == TPDF_Rejection_Reasons.PDF417_DISCONNECTED)
                rrEdit.Text = "Decoder Disconnected";
            else if (Decoder.RejectionReasons() == TPDF_Rejection_Reasons.PDF417_RS_Error)
                rrEdit.Text = "Reed Solomon Error";
            else if (Decoder.RejectionReasons() == TPDF_Rejection_Reasons.PDF417_RowCount_Error)
               rrEdit.Text = "RowCount Error";
            else 
                rrEdit.Text = "PDF not Found";

            CountEdit.Text =  Decoder.SymbolCount().ToString();

            if (Decoder.SymbolCount() > 0)
            {
                SymbolNumber.Enabled = true;
                SymbolNumber.Maximum = Decoder.SymbolCount() - 1;
                ShowInfo(0);
            }
            SymbolNumber.Focus();
        }

        public void ClearForm()
        {
            textBox1.Text = "";
            TB_symbol_contrast.Text = "";
            TB_min_reflectance.Text = "";
            TB_defects.Text = "";
            TB_max_reflectance.Text = "";
            TB_unused_error_correction.Text = "";
            TB_decodability.Text = "";
            TB_decodability_grade.Text = "";

            TB_symbol_contrast_grade.Text = "";
            TB_min_reflectance_grade.Text = "";
            TB_defects_grade.Text = "";
            TB_unused_error_correction_grade.Text = "";
            TB_modulation_grade.Text = "";
            TB_decode_grade.Text = "";
            TB_overall_grade.Text = "";

            TimeEdit.Text = "";
            CountEdit.Text = "";
            rrEdit.Text = "";
            SymbolNumber.Enabled = false;
            SymbolNumber.Value = 0;
            SymbolNumber.Maximum = 0;
        }
        
        private void button1_Click(object sender, EventArgs e)
        {
            ClearForm();
            DialogResult R = OpenDialog.ShowDialog( );
            if ( R == DialogResult.OK )
            {
                SourcePicture = Image.FromFile( OpenDialog.FileName );
                Img.Image = SourcePicture;
                
                LabelFileName.Text = OpenDialog.FileName;
                button2.Enabled = true;
            }

        }

        private void ShowInfo (int Index)
        {
            for (int i = 0; i < Decoder.SymbolCount(); i++)
            {
               bool Ok = (i == Index) && (Decoder.GetSymbolRSErrorCount(i) >= 0);
                DrawCorners( Ok, Decoder.GetSymbolRect(i) );
            }
            InfoToForm(Index);
        }
        String QValue (float V)
        {
//            if (V < 0) return "";
            return V.ToString("0.00");
        }
        String QGrade (float V)
        {
            if ( V < 0 ) return "";
            return V.ToString(  );
        }
        
        private void InfoToForm (int symbolIndex)
        {
           if (symbolIndex < 0)
            {
                ClearForm();
            }
            else
            {
                String[] no_yes = { "No", "Yes" };
                
                textBox1.Text = Decoder.GetSymbolData(CodePage, symbolIndex);

                TPDF417_Quality Q = Decoder.GetSymbolQuality(symbolIndex);

                TB_symbol_contrast.Text = QValue(Q.symbol_contrast);
                TB_min_reflectance.Text = QValue(Q.min_reflectance);
                TB_max_reflectance.Text = QValue(Q.max_reflectance);
                TB_unused_error_correction.Text = QValue( Q.unused_error_correction );
                TB_decodability.Text = QValue(Q.decodability);
                TB_defects.Text = QValue(Q.defects);

                TB_symbol_contrast_grade.Text = QGrade(Q.symbol_contrast_grade);
                TB_min_reflectance_grade.Text = QGrade(Q.min_reflectance_grade);
                TB_defects_grade.Text = QGrade( Q.defects_grade );
                TB_unused_error_correction_grade.Text = QGrade( Q.unused_error_correction_grade );
                TB_modulation_grade.Text = QGrade( Q.modulation_grade );
                TB_decode_grade.Text = QGrade( Q.decode_grade );
                TB_overall_grade.Text = QGrade( Q.overall_grade );
                TB_decodability_grade.Text = QGrade(Q.decodability_grade);

            }
            
        }
        private void SymbolNumber_ValueChanged (object sender, EventArgs e)
        {
            int Index = (int)SymbolNumber.Value ;
            symbolIndex = Index;
            ShowInfo( Index );
        }

        private void comboBox1_SelectedIndexChanged (object sender, EventArgs e)
        {
            int Index = comboBox1.SelectedIndex;
            CodePage = Info[ Index ].CodePage;

            if ((Decoder != null) && (Decoder.SymbolCount() > 0))
               textBox1.Text = Decoder.GetSymbolData(CodePage, symbolIndex);
        }

        private void Img_MouseDown(object sender, MouseEventArgs e)
        {
            int col, row;
            int i, n;
            int i0;
            int s0, s1, s;
            bool confirm;
            int rowcount = Picture.Height;
            int colcount = Picture.Width;
            
            int r, w, c, h, dw, dh;

            c = e.X;
            r = e.Y;
            w = Img.Width;
            h = Img.Height;

            dw = dh = 0;
            if (rowcount * w  > colcount * h){ //w/h > colcount/rowcount => Vertical stretch
                dw = (w-colcount*h/rowcount)/2;
            }
            else if (rowcount * w  < colcount * h) // horizontal stretch
            {
                dh = (h-rowcount*w/colcount)/2;
            }
            if (dw>0)
//                col = (c - dw) * colcount / (colcount*h/rowcount) ;
                col = (c - dw) * rowcount / h;
            else
                col = c * colcount / w;
            if (dh>0)
                row = (r - dh) * colcount / w;
            else
                row = r * rowcount / h;

            i0 = -1;
            s1 = 0x7FFFFFFF;
            int count = Decoder.SymbolCount();
            int[] RC;
            for (i = 0; i < count; i++)
            {
               RC = Decoder.GetSymbolRect(i);
                s0 = (int)((RC[0] - row) * (RC[3] - col) -
                           (RC[1] - col) * (RC[2] - row));
                confirm = true;
                for (n = 1; n < 4; n++)
                {
                    s = (int)((RC[2 * n % 8] - row) * (RC[(2 * n + 3) % 8] - col) -
                        (RC[(2 * n + 1) % 8] - col) * (RC[(2 * n + 2) % 8] - row));
                    if ((s0 > 0) && (s < 0)) confirm = false;
                    if ((s0 < 0) && (s > 0)) confirm = false;
                }
                if (confirm) // point of "mouse down" belongs to rectangle
                {
                    SymbolNumber.Value = i; // change matrix to "i"
                    return;
                }
                for (n = 0; n < 4; n++) // looking for closest corner
                {
                    s = (int)((RC[2 * n] - row) * (RC[2 * n] - row) +
                              (RC[2 * n + 1] - col) * (RC[2 * n + 1] - col));
                    if (s1 > s)
                    {
                        s1 = s;
                        i0 = i;
                    }
                }
            }
            if (i0 >= 0)
            {
                SymbolNumber.Value = i0;
                return;
            }
        }

        private void MaxPDFCountEdit_ValueChanged(object sender, EventArgs e)
        {
           optionsChanged = true;
        }

        private void SpeedComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
           optionsChanged = true;
        }

        private void DirectComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
           optionsChanged = true;
        }

        private void stepNumericUpDown1_ValueChanged(object sender, EventArgs e)
        {
           optionsChanged = true;
        }

        private void QualityComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
           optionsChanged = true;
        }

        private void bwFilterComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
           optionsChanged = true;
        }

        private void TimeOutEdit_ValueChanged(object sender, EventArgs e)
        {
           optionsChanged = true;
        }

        private void label32_Click(object sender, EventArgs e)
        {
                   }

    }
}